package org.apache.coyote;

import java.util.List;
import java.util.Map.Entry;

import org.apache.catalina.startup.ContextConfig;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.json.JettisonMappedXmlDriver;

/**
 * helps stuff marshal and unmarshal with json and xml.
 */
public class JsonXmlHelper {

	private XStream xJSON;
	private XStream xXML;

	private ClassLoader classLoader;

	public JsonXmlHelper(ClassLoader classLoader) {
		this.classLoader = classLoader;

		createJson();
		createXml();
	}

	private void createXml() {
		xXML = new XStream();
		xXML.setClassLoader(classLoader);
		xXML.setMode(XStream.NO_REFERENCES);
		for(Entry<String, String> e : ContextConfig.getXmlbindings().entrySet()){
			try {
				xXML.alias(e.getKey(), classLoader.loadClass(e.getValue()));
			} catch (ClassNotFoundException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}

	private void createJson() {
		xJSON = new XStream(new JettisonMappedXmlDriver());
		xJSON.setClassLoader(classLoader);
		xJSON.setMode(XStream.NO_REFERENCES);
		xJSON.alias("list", List.class);
		for(Entry<String, String> e : ContextConfig.getJsonbindings().entrySet()){
			try {
				xJSON.alias(e.getKey(), classLoader.loadClass(e.getValue()));
			} catch (ClassNotFoundException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}
	
	protected String marshalJson(Object o){
		return xJSON.toXML(o);
	}
	
	protected String marshalXml(Object o){
		return xXML.toXML(o);
	}
	
	protected Object unmarshalJson(String s){
		return xJSON.fromXML(s);
	}
	
	protected Object unmarshalXml(String s){
		return xXML.fromXML(s);
	}
	
}
