/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */


package org.apache.coyote.http11;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.apache.coyote.Request;

/**
 * Implementation of InputBuffer which is specific to websockets.
 *
 * @author Ant Kutschera
 */
public class WebSocketsInternalNioInputBuffer extends InternalNioInputBuffer {

    /**
     * Logger.
     */
    private static final org.apache.juli.logging.Log log =
        org.apache.juli.logging.LogFactory.getLog(WebSocketsInternalNioInputBuffer.class);

    /**
     * constructor.
     */
    public WebSocketsInternalNioInputBuffer(Request request, int headerBufferSize) {
    	super(request, headerBufferSize);
    }

    // --------------------------------------------------------- Public Methods
    
    public boolean parseThirdKey() {
    	ByteArrayOutputStream baos = new ByteArrayOutputStream();
		while(true){
			byte[] bs = new byte[1];
			bs[0] = buf[pos];

			if(pos >= lastValid){
				break;
			}
			if ((bs[0] != Constants.CR) && (bs[0] != Constants.LF)) {
				try {
					baos.write(bs);
				} catch (IOException e) {
					//will never ever happen!
					throw new RuntimeException(e);
				}
			}
			
			pos++;
		}
		
        byte[] thirdKey = baos.toByteArray();
        
        //should be 8 long...
        if(thirdKey.length != 8){
        	//hmmm...
        	//let it fall through, because the handshake will fail anyway
        	log.warn("third key of websocket request was not 8 bytes... " + thirdKey);
        }
        
        request.setWebSocketsThirdKey(thirdKey);
        return true;
    }

    /** added by ant, becuase otherwise, the void filter that we have on our websockets buffer ruins the reading of websocket frames */
	public void clearFilters() {
		this.lastActiveFilter = -1;
	}
}
