/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package org.apache.coyote.websockets;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Map;

import javax.servlet.AsyncContext;
import javax.servlet.DispatcherType;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.websockets.WebSocketsServletRequest;
import javax.servlet.websockets.WebSocketsServletResponse;
import javax.servlet.websockets.WebSocketsSession;

import org.apache.catalina.comet.CometEvent;
import org.apache.coyote.JsonXmlHelper;

/** the container specific impl of the {@link WebSocketsServletRequest} */
public class WebSocketsServletRequestImpl<T> extends JsonXmlHelper implements WebSocketsServletRequest<T> {

	protected static final String DEFAULT_ENCODING = "UTF-8";
	protected static final String SEC_WEB_SOCKET_PROTOCOL = "Sec-WebSocket-Protocol";

	private final HttpServletRequest httpServletRequest;
	private byte[] data;
	private T object;
	private String text;

	public WebSocketsServletRequestImpl(CometEvent event){
		super(event.getHttpServletRequest().getServletContext().getClassLoader());
		this.httpServletRequest = event.getHttpServletRequest();
	}
	
	@Override
	public boolean authenticate(WebSocketsServletResponse response)
			throws IOException, ServletException {
		
		return httpServletRequest.authenticate(((WebSocketsServletResponseImpl)response).httpServletResponse);
	}
	
	@Override
	public AsyncContext getAsyncContext() {
		return this.httpServletRequest.getAsyncContext();
	}
	
	@Override
	public Object getAttribute(String name) {
		return httpServletRequest.getAttribute(name);
	}
	
	@Override
	public Enumeration<String> getAttributeNames() {
		return httpServletRequest.getAttributeNames();
	}
	
	@Override
	public String getAuthType() {
		return httpServletRequest.getAuthType();
	}
	
	@Override
	public String getCharacterEncoding() {
		return httpServletRequest.getCharacterEncoding();
	}
	
	@Override
	public int getContentLength() {
		return -1;
	}
	
	@Override
	public String getContentType() {
		return httpServletRequest.getContentType();
	}
	
	@Override
	public String getContextPath() {
		return httpServletRequest.getContextPath();
	}
	
	@Override
	public Cookie[] getCookies() {
		return httpServletRequest.getCookies();
	}
	
	@Override
	public long getDateHeader(String name) {
		return httpServletRequest.getDateHeader(name);
	}
	
	@Override
	public DispatcherType getDispatcherType() {
		return null;
	}
	
	@Override
	public String getHeader(String name) {
		return httpServletRequest.getHeader(name);
	}
	
	@Override
	public Enumeration<String> getHeaderNames() {
		return httpServletRequest.getHeaderNames();
	}
	
	@Override
	public Enumeration<String> getHeaders(String name) {
		return httpServletRequest.getHeaders(name);
	}
	
	@Override
	public ServletInputStream getInputStream() throws IOException {
		throw new UnsupportedOperationException("not for use with websockets - websocket servlets are notified of incoming data in their doXXX methods");
	}
	
	@Override
	public int getIntHeader(String name) {
		return httpServletRequest.getIntHeader(name);
	}
	
	@Override
	public String getLocalAddr() {
		return httpServletRequest.getLocalAddr();
	}
	
	@Override
	public Locale getLocale() {
		return httpServletRequest.getLocale();
	}
	
	@Override
	public Enumeration<Locale> getLocales() {
		return httpServletRequest.getLocales();
	}
	
	@Override
	public String getLocalName() {
		return httpServletRequest.getLocalName();
	}
	
	@Override
	public int getLocalPort() {
		return httpServletRequest.getLocalPort();
	}
	
	@Override
	public String getMethod() {
		return httpServletRequest.getMethod();
	}
	
	@Override
	public String getParameter(String name) {
		return httpServletRequest.getParameter(name);
	}
	
	@Override
	public Map<String, String[]> getParameterMap() {
		return httpServletRequest.getParameterMap();
	}
	
	@Override
	public Enumeration<String> getParameterNames() {
		return httpServletRequest.getParameterNames();
	}
	
	@Override
	public String[] getParameterValues(String name) {
		return httpServletRequest.getParameterValues(name);
	}
	
	@Override
	public String getPathInfo() {
		return httpServletRequest.getPathInfo();
	}
	
	@Override
	public String getPathTranslated() {
		return httpServletRequest.getPathTranslated();
	}
	
	@Override
	public String getProtocol() {
		return httpServletRequest.getProtocol();
	}
	
	@Override
	public String getQueryString() {
		return httpServletRequest.getQueryString();
	}
	
	@Override
	public BufferedReader getReader() throws IOException {
		throw new UnsupportedOperationException("websockets read data using their doXXX methods!");
	}

	@Override
	public String getRealPath(String path) {
		throw new UnsupportedOperationException("deprecated anyway!");
	}
	
	@Override
	public String getRemoteAddr() {
		return httpServletRequest.getRemoteAddr();
	}
	
	@Override
	public String getRemoteHost() {
		return httpServletRequest.getRemoteHost();
	}
	
	@Override
	public int getRemotePort() {
		return httpServletRequest.getRemotePort();
	}
	@Override
	
	public String getRemoteUser() {
		return httpServletRequest.getRemoteUser();
	}
	
	@Override
	public RequestDispatcher getRequestDispatcher(String path) {
		throw new UnsupportedOperationException("dont work with websockets paradigm!");
	}
	
	@Override
	public String getRequestedSessionId() {
		return httpServletRequest.getRequestedSessionId();
	}
	
	@Override
	public String getRequestURI() {
		return httpServletRequest.getRequestURI();
	}
	
	@Override
	public StringBuffer getRequestURL() {
		return httpServletRequest.getRequestURL();
	}
	
	@Override
	public String getScheme() {
		return httpServletRequest.getScheme();
	}
	
	@Override
	public String getServerName() {
		return httpServletRequest.getServerName();
	}
	
	@Override
	public int getServerPort() {
		return httpServletRequest.getServerPort();
	}
	
	@Override
	public ServletContext getServletContext() {
		return httpServletRequest.getServletContext();
	}
	
	@Override
	public String getServletPath() {
		return httpServletRequest.getServletPath();
	}
	
	@Override
	public WebSocketsSession getSession() {
		return new WebSocketsSessionImpl(httpServletRequest.getSession());
	}
	
	@Override
	public WebSocketsSession getSession(boolean create) {
		return new WebSocketsSessionImpl(httpServletRequest.getSession(create));
	}
	
	@Override
	public Principal getUserPrincipal() {
		return httpServletRequest.getUserPrincipal();
	}
	
	@Override
	public boolean isAsyncStarted() {
		return httpServletRequest.isAsyncStarted();
	}
	
	@Override
	public boolean isAsyncSupported() {
		return httpServletRequest.isAsyncSupported();
	}
	
	@Override
	public boolean isRequestedSessionIdFromCookie() {
		return httpServletRequest.isRequestedSessionIdFromCookie();
	}
	
	@Override
	public boolean isRequestedSessionIdFromURL() {
		return httpServletRequest.isRequestedSessionIdFromURL();
	}
	
	@Override
	public boolean isRequestedSessionIdValid() {
		return httpServletRequest.isRequestedSessionIdValid();
	}
	
	@Override
	public boolean isSecure() {
		return httpServletRequest.isSecure();
	}
	
	@Override
	public boolean isUserInRole(String role) {
		return httpServletRequest.isUserInRole(role);
	}
	
	@Override
	public void login(String username, String password) throws ServletException {
		httpServletRequest.login(username, password);
	}
	
	@Override
	public void logout() throws ServletException {
		httpServletRequest.logout();
	}
	
	@Override
	public void removeAttribute(String name) {
		httpServletRequest.removeAttribute(name);
	}
	
	@Override
	public void setAttribute(String name, Object o) {
		httpServletRequest.setAttribute(name, o);
	}
	
	@Override
	public void setCharacterEncoding(String env)
			throws UnsupportedEncodingException {
		httpServletRequest.setCharacterEncoding(env);
	}
	
	@Override
	public AsyncContext startAsync() {
		return httpServletRequest.startAsync();
	}
	
	@Override
	public AsyncContext startAsync(ServletRequest servletRequest,
			ServletResponse servletResponse) {
		return httpServletRequest.startAsync(servletRequest, servletResponse);
	}
	
	/** used for websockets - this is the frame 
	 * @throws Exception */
	@SuppressWarnings("unchecked")
	@Override
	public void setData(byte[] data) {
		
		try{
			this.text = new String(data, getCharEnc());
		}catch(UnsupportedEncodingException e){
			//ugly
		}
		
		if(this.text != null){
			try{
				this.object = (T) unmarshalJson(this.text);
			}catch(Exception e){
				try{
					this.object = (T) unmarshalXml(this.text);
				}catch(Exception e2){
					System.out.print("");
					//ok, thats bad... must be binary data
				}
			}
		}
		
		this.data = data;
	}
	
	private String getCharEnc() {
		String charEnc = getCharacterEncoding();
		if(charEnc == null){
			charEnc = DEFAULT_ENCODING;
		}
		return charEnc;
	}

	public T getDataObject() {
		return this.object;
	}

	@Override
	public String getDataText() {
		return this.text;
	}
	
	@Override
	public byte[] getBinary() {
		return data;
	}
	
	@Override
	public String getWebSocketsProtocol() {
		String protocol = getHeader(SEC_WEB_SOCKET_PROTOCOL);
		if(protocol == null){
			protocol = TEXT;
		}
		return protocol;
	}

}
