package org.apache.coyote.websockets;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Locale;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.websockets.WebSocketsServletResponse;

import org.apache.catalina.comet.CometEvent;
import org.apache.coyote.JsonXmlHelper;

public class WebSocketsServletResponseImpl extends JsonXmlHelper implements WebSocketsServletResponse {
	
	/*package*/ final HttpServletResponse httpServletResponse;
	private CometEvent event;

	public WebSocketsServletResponseImpl(CometEvent event) {
		super(event.getHttpServletRequest().getServletContext().getClassLoader());
		this.httpServletResponse = event.getHttpServletResponse();
		this.event = event;
	}

	@Override
	public String getCharacterEncoding() {
		return this.httpServletResponse.getCharacterEncoding();
	}

	@Override
	public String getContentType() {
		return this.httpServletResponse.getContentType();
	}

	@Override
	public ServletOutputStream getOutputStream() throws IOException {
		throw new UnsupportedOperationException("use send instead!");
	}

	@Override
	public PrintWriter getWriter() throws IOException {
		throw new UnsupportedOperationException("use send instead!");
	}

	@Override
	public void setCharacterEncoding(String charset) {
		this.httpServletResponse.setCharacterEncoding(charset);
	}

	@Override
	public void setContentLength(int len) {
		throw new UnsupportedOperationException("not relevant to websockets at the moment!");
	}

	@Override
	public void setContentType(String type) {
		this.httpServletResponse.setContentType(type);
	}

	@Override
	public void setBufferSize(int size) {
		throw new UnsupportedOperationException("not relevant to websockets at the moment!");
	}

	@Override
	public int getBufferSize() {
		throw new UnsupportedOperationException("not relevant to websockets at the moment!");
	}

	@Override
	public void flushBuffer() throws IOException {
		this.httpServletResponse.flushBuffer();
	}

	@Override
	public void resetBuffer() {
		this.httpServletResponse.resetBuffer();
	}

	@Override
	public boolean isCommitted() {
		return true;
	}

	@Override
	public void reset() {
		throw new UnsupportedOperationException("not relevant to websockets at the moment!");
	}

	@Override
	public void setLocale(Locale loc) {
		this.httpServletResponse.setLocale(loc);
	}

	@Override
	public Locale getLocale() {
		return this.httpServletResponse.getLocale();
	}

	@Override
	public String encodeURL(String url) {
		return this.httpServletResponse.encodeURL(url);
	}

	@Override
	public void sendJSONResponse(Object data) throws IOException {
		String json = marshalJson(data);
		sendTextResponse(json);
	}

	@Override
	public void sendXMLResponse(Object data) throws IOException {
		String xml = marshalXml(data);
		sendTextResponse(xml);
	}

	@Override
	public void sendTextResponse(String text) throws IOException {
		byte[] data = text.getBytes(getCharEnc());
		sendBinaryResponse(data);
	}

	@Override
	public void sendBinaryResponse(byte[] data) throws IOException {
		byte[] frame = new byte[data.length + 2];
		frame[0] = 0;
		frame[frame.length - 1] = -1;
		System.arraycopy(data, 0, frame, 1, data.length);
		this.httpServletResponse.getOutputStream().write(frame);
		this.httpServletResponse.getOutputStream().flush();
	}

	@Override
	public void close() throws IOException {
		event.close();
	}

	private String getCharEnc(){
		String charEnc = getCharacterEncoding();
		if(charEnc == null){
			charEnc = WebSocketsServletRequestImpl.DEFAULT_ENCODING;
		}
		return charEnc;
	}
	
}
